/*
 * Copyright (c) 2013 CNRS/LAAS
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <pthread.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <termios.h>
#include <unistd.h>

#include <rmp440/rmp440.h>

struct rmp440_io *rmp;
struct rmp440_feedback *fb;
double v = 0;
int done = 0;

static void *
rmpTask(void *arg)
{
	struct timespec tv;

	while (!done) {
		tv.tv_sec = 0;
		tv.tv_nsec = 10000000; /* 10ms */
		nanosleep(&tv, NULL);
		switch (fb->operational_state) {
		case 0:
		case 1:
		case 2:
			rmp440CmdNone(rmp);
			break;
		case 3:/* standby */
			if (v > -10.0)
				rmp440SetOperationalMode(rmp,
				    RMP_TRACTOR_REQUEST);
			else
				rmp440CmdNone(rmp);
			break;
		case 4: /* tractor */
			if (v >= -1.0)
				rmp440Motion(rmp, v, 0);
			else
				rmp440SetOperationalMode(rmp,
				    RMP_STANDBY_REQUEST);
			break;
		}
		if (rmp440ReceiveAndDecode(rmp, fb) < 0) {
			printf("Communication error with rmp440\n");
			done = 1;
		}
	}
	rmp440SetOperationalMode(rmp, RMP_STANDBY_REQUEST);
	rmp440End(rmp);
	return NULL;
}

int
main(int argc, char *argv[])
{
	pthread_t tid;
	char *buf = NULL;
	size_t n;

	fb = rmp440FeedbackInit(NULL);

	rmp = rmp440Init(RMP440_INTERFACE_ETHERNET, "10.40.40.40:8080");
	if (rmp == NULL) {
		fprintf(stderr, "Cannot init rmp connection\n");
		exit(2);
	}
	pthread_create(&tid, NULL, rmpTask, NULL);

	usleep(500000);

	printf("sp_sw_build_id: %d\n", fb->sp_sw_build_id);
	printf("uip_sw_build_id: %d\n", fb->uip_sw_build_id);

	while (!done) {
		printf("%s ",
		    rmp440DecodeOperationalState(fb->operational_state));
		printf("frame count: %f\n", fb->frame_count);
		printf("target vel:  %f\n", fb->vel_target);
		printf("aux battery: %f %% %f V %f A\n", fb->aux_batt_soc,
		    fb->aux_batt_voltage, fb->aux_batt_current);
		printf("main batteries: %f\n", fb->min_propulsion_batt_soc);
		printf("linear_vel: %f\n", fb->linear_vel);
		printf("differential_wheel_vel: %f\n",
		    fb->differential_wheel_vel);

		fflush(stdin);
		tcflush(0, TCIFLUSH);
		printf("v> ");
		if (getline(&buf, &n, stdin) < 0)
			done = 1;
		if (buf[0] == 'q' || buf[0] == 'Q')
			done = 1;
		if (!done)
			if (sscanf(buf, "%lf", &v) != 1)
				v = 0.0;
	}
	pthread_join(tid, NULL);
	exit(0);
}
